@@ -33,24 +33,24 @@
  * memchunks, to using malloc/realloc/free */
 #define RBUF_MEMCHUNK_SIZE 1024
 
-/* Should have proper warning system for this type of thing.. */
-#define RBUF_RETURN_IF_RDONLY(buf) \
-if (rbuf_rdonly(buf)) { \
-    printf ("ACK! Someone tried to mess with a rdonly rbuf!\n"); \
-        return; \
-}
 
 
 #define rbuf_maybe_expand(buf, size) \
     do { \
         unsigned int roy_len_total = buf->len + size; \
         if ((roy_len_total >= buf->alloc) || roy_len_total < (size >> 3)) \
-            rbuf_resize(buf, size); \
+            rbuf_resize (buf, size); \
     } while (0);
 
+#define RBUF_FREE_BUF(buf) \
+    if (buf->alloc < RBUF_MEMCHUNK_SIZE) { \
+        rchunk_free (buf->str, buf->alloc); \
+    } else { \
+        rmem_free (buf->str); \
+    }
 
 static void
-rbuf_resize (RBuf * buf, unsigned int size)
+rbuf_resize (RBuf *buf, unsigned int size)
 {
     unsigned int total;
     unsigned int new_size = RBUF_MIN_SIZE;
@@ -67,7 +67,7 @@ rbuf_resize (RBuf * buf, unsigned int size)
         return;
 
     /* Deal with memchunk allocated strings */
-    if (total < RBUF_MEMCHUNK_SIZE) {
+    if (new_size < RBUF_MEMCHUNK_SIZE) {
 
         /* create a new buffer with rchunk */
         tmpbuf = rchunk_alloc (new_size);
@@ -77,12 +77,9 @@ rbuf_resize (RBuf * buf, unsigned int size)
             if (buf->len > 0) {
                 memcpy (tmpbuf, buf->str, buf->len);
             }
-
-            /* Free the old buffer if necessary */
-            if (buf->alloc < RBUF_MEMCHUNK_SIZE)
-                rchunk_free (buf->str, buf->alloc);
-            else
-                rmem_free (buf->str);
+       
+            /* Free old buffer */
+            RBUF_FREE_BUF (buf);
         }
 
         buf->str = tmpbuf;
@@ -90,7 +87,9 @@ rbuf_resize (RBuf * buf, unsigned int size)
     } else {
         /* Total is too big to use a memchunk */
 
-        if (buf->alloc > RBUF_MEMCHUNK_SIZE) {
+        /* We do this inverse logic so we can be certain it matches
+         * the logic above so we don't get off-by-one errors */
+        if (!(buf->alloc < RBUF_MEMCHUNK_SIZE)) {
             /* if the old size was also from an malloc, try doing
              * realloc and hope the system can just expand the 
              * block */
@@ -98,15 +97,14 @@ rbuf_resize (RBuf * buf, unsigned int size)
         } else {
             /* we are moving from a previous rchunk_alloc to rmem_alloc
              * style */
-
-            tmpbuf = rmem_alloc (new_size);\
-                if (buf->str) {
-                    if (buf->len > 0) {
-                        memcpy (tmpbuf, buf->str, buf->len);
-                    }
-                    rchunk_free (buf->str, buf->alloc);
+            tmpbuf = rmem_alloc (new_size);
+            if (buf->str) {
+                if (buf->len > 0) {
+                    memcpy (tmpbuf, buf->str, buf->len);
                 }
-
+                /* Free old buffer */
+                RBUF_FREE_BUF (buf);
+            }
             buf->str = tmpbuf;
         }
     }
@@ -123,11 +121,7 @@ rbuf_free (RBuf *buf)
     }
 
     if (!RFLAG_ISSET (buf, RBUF_STATIC)) {
-        if (buf->alloc < RBUF_MEMCHUNK_SIZE) {
-            rchunk_free (buf->str, buf->alloc);
-        } else {
-            rmem_free (buf->str);
-        }
+        RBUF_FREE_BUF (buf);
     }
 
     rchunk_free (buf, sizeof (RBuf));
@@ -213,7 +207,7 @@ rbuf_new_with_data (const char *str, unsigned int len)
 }
 
 RBuf *
-rbuf_new_with_rbuf (const RBuf * buf)
+rbuf_new_with_rbuf (const RBuf *buf)
 {
     RBuf *newbuf;
 
@@ -524,7 +518,7 @@ rbuf_erase (RBuf *buf, unsigned int pos, unsigned int len)
     RBUF_RETURN_IF_RDONLY (buf);
 
     memmove (buf->str + pos, buf->str + pos + len,
-        buf->len - (pos + len));
+             buf->len - (pos + len));
     buf->len -= len;
     buf->str[buf->len] = '\0';
 }
@@ -643,21 +637,85 @@ rbuf_equal_rbuf (const RBuf *buf1, const RBuf *buf2)
 }
 
 unsigned int
-rbuf_equal_str (const RBuf *buf1, const char *str)
+rbuf_equal_data (const RBuf *buf, const char *str, unsigned int len)
 {
-    if (!buf1 || !str)
+    if (!buf || !str)
         return (FALSE);
 
-    return (strcmp (buf1->str, str) == 0);
+    return (strncmp (buf->str, str, len) == 0);
 }
 
 unsigned int
-rbuf_equal_strcase (const RBuf *buf1, const char *str)
+rbuf_equal_rbuf_len (const RBuf *buf1, const RBuf *buf2, unsigned int len)
 {
-    if (!buf1 || !str)
+    unsigned char *c1;
+    unsigned char *c2;
+    unsigned int *i1;
+    unsigned int *i2;
+    unsigned int i;
+    unsigned int num_ints;
+
+    /* First we see if both strings are longer than the len passed
+     * in by the user. */
+    if ((rbuf_len (buf1) < len) || (rbuf_len (buf2) < len)) {
+        /* One or both are shorter so they have to be the same length ot match. */
+        if (rbuf_len (buf1) != rbuf_len (buf2))
+            return (FALSE);
+        /* They are both short, but the same length, so we set len to their length. */
+        len = rbuf_len (buf1);
+    }
+   
+    i1 = (unsigned int *) rbuf_str (buf1);
+    i2 = (unsigned int *) rbuf_str (buf2);
+
+    num_ints = len / sizeof (int);
+    for (i = num_ints; i--; i1++, i2++) {
+	if (*i1 != *i2)
+	    return (FALSE);
+    }
+   
+    c1 = (unsigned char *) i1;
+    c2 = (unsigned char *) i2;
+
+    /* c1 and c2 now point to the last bit of the string, 
+     * so we can just assign our pointers to them, and compare
+     * from there */
+    i = len - (num_ints * sizeof (int));
+
+    for (; i--; c1++, c2++) {
+	if (*c1 != *c2)
+	    return (FALSE);
+    }
+
+    return (TRUE);
+}
+
+unsigned int
+rbuf_equal_str_len (const RBuf *buf, const char *str, unsigned int len)
+{
+    if (!buf || !str)
         return (FALSE);
 
-    return (strcasecmp (buf1->str, str) == 0);
+    return (strncmp (rbuf_str (buf), str, len) == 0);
+}
+
+
+unsigned int
+rbuf_equal_str (const RBuf *buf, const char *str)
+{
+    if (!buf || !str)
+        return (FALSE);
+
+    return (strcmp (rbuf_str (buf), str) == 0);
+}
+
+unsigned int
+rbuf_equal_strcase (const RBuf *buf, const char *str)
+{
+    if (!buf || !str)
+        return (FALSE);
+
+    return (strcasecmp (rbuf_str (buf), str) == 0);
 }
 
 /* A slightly faster, cheap knock off */
@@ -666,9 +724,9 @@ rbuf_hash2 (const RBuf *buf)
 {
     unsigned long hashval = 0;
 
-    if (rbuf_len(buf) >= sizeof (int)) {
+    if (rbuf_len (buf) >= sizeof (int)) {
 	const unsigned int *i;
-        unsigned int num_ints = rbuf_len(buf) / sizeof(int);
+        unsigned int num_ints = rbuf_len (buf) / sizeof (int);
 
         for (i = ((unsigned int *) rbuf_str (buf)); num_ints--; i++) {
             hashval += (hashval << 3) + *i;
@@ -760,22 +818,27 @@ typedef struct
 static RList *rbuf_auto_list_entries = NULL;
 static RBufAutoListEntry *rbuf_auto_list_entry = NULL;
 
+static RThreadMutex rbuf_auto_list_entries_mutex;
+
 #define RBUF_AUTO_NUM_BUFS 32
 
-void
-rbuf_auto_cleanup__P (void);
 
-
-void
-rbuf_auto_cleanup__P (void)
+static void
+rbuf_auto_cleanup (void)
 {
 
-    /* We do it reverse because it's a circular list going forward */
-    RLIST_FOREACH_REVERSE (rbuf_auto_list_entries, rbuf_auto_list_entry) {
-        rchunk_free (rbuf_auto_list_entry, sizeof (RBufAutoListEntry));
-    } RFOREACH_CLOSE;
+    RTHREAD_MUTEX_ENTER (rbuf_auto_list_entries_mutex) {
+        
+        /* We do it reverse because it's a circular list going forward */
+        RLIST_FOREACH_REVERSE (rbuf_auto_list_entries, rbuf_auto_list_entry) {
+            rchunk_free (rbuf_auto_list_entry, sizeof (RBufAutoListEntry));
+        } RFOREACH_CLOSE;
 
-    rlist_free (rbuf_auto_list_entries);
+        rlist_free (rbuf_auto_list_entries);
+
+    } RTHREAD_MUTEX_EXIT (rbuf_auto_list_entries_mutex);
+    
+    rthread_mutex_destroy (rbuf_auto_list_entries_mutex);
 }
 
 void
@@ -803,7 +866,9 @@ rbuf_auto_init__P (void)
         rbuf_auto_list_entry = rlist_first (rbuf_auto_list_entries);
     }
 
-    rcleanup_add_hook (rbuf_auto_cleanup__P);
+    rbuf_auto_list_entries_mutex = rthread_mutex_new ();
+
+    rcleanup_add_hook (rbuf_auto_cleanup);
 }
 
 RBuf *
@@ -811,8 +876,12 @@ rbuf_auto (char *str)
 {
     RBuf *newbuf;
 
-    newbuf = &rbuf_auto_list_entry->buf;
-    rbuf_auto_list_entry = rlist_next (rbuf_auto_list_entry);
+    RTHREAD_MUTEX_ENTER (rbuf_auto_list_entries_mutex) {
+        
+        newbuf = &rbuf_auto_list_entry->buf;
+        rbuf_auto_list_entry = rlist_next (rbuf_auto_list_entry);
+    
+    } RTHREAD_MUTEX_EXIT (rbuf_auto_list_entries_mutex);
 
     newbuf->str = str;
     newbuf->len = strlen (str);
@@ -823,3 +892,83 @@ rbuf_auto (char *str)
 
 
 
+/* sprintf style stuff with rbufs.
+ * This all uses the trio library which I've incldued with roy
+ * for convenience.
+ *
+ * Only thing I had to change was to make it use roy-config.h
+ * instead of config.h 
+ */
+
+#include "trio.h"
+
+
+static int
+rbuf_trio_stream_out (void *data, int c)
+{
+    RBuf *buf = data;
+    
+    rbuf_append_char (buf, (unsigned char) c);
+
+    return (1);
+}
+
+
+void
+rbuf_append_vsprintf (RBuf *buffer, const char *format, va_list args)
+{
+    int status;
+
+    RBUF_RETURN_IF_RDONLY (buffer);
+
+    status = trio_vcprintf (rbuf_trio_stream_out, buffer, format, args);
+}
+
+
+void
+rbuf_sprintf (RBuf *buffer, const char *format, ...)
+{
+    int status;
+    va_list args;
+
+    RBUF_RETURN_IF_RDONLY (buffer);
+
+    rbuf_truncate (buffer, 0);
+
+    TRIO_VA_START(args, format);
+    status = trio_vcprintf (rbuf_trio_stream_out, buffer, format, args);
+    TRIO_VA_END(args);
+}
+
+void
+rbuf_append_sprintf (RBuf *buffer, const char *format, ...)
+{
+    int status;
+    va_list args;
+
+    RBUF_RETURN_IF_RDONLY (buffer);
+
+    TRIO_VA_START(args, format);
+    status = trio_vcprintf (rbuf_trio_stream_out, buffer, format, args);
+    TRIO_VA_END(args);
+}
+
+RBuf *
+rbuf_new_with_sprintf (const char *format, ...)
+{
+    RBuf *buffer;
+    int status;
+    va_list args;
+
+    buffer = rbuf_new ();
+
+    TRIO_VA_START(args, format);
+    status = trio_vcprintf (rbuf_trio_stream_out, buffer, format, args);
+    TRIO_VA_END(args);
+
+    return (buffer);
+}
+
+
+
+
