@@ -26,11 +26,34 @@
 
 #include <roy.h>
 
+static int roy_initialized = FALSE;
+
 void
 rinit (void)
 {
-    rbuf_auto_init__P ();
-    rbuf_intern_init__P ();
+    if (!roy_initialized) {
+        roy_initialized = TRUE;
+
+        rchunk_init__P ();
+        rbuf_auto_init__P ();
+        rthread_init__P ();
+
+        rthread_enabled = FALSE;
+    }
+}
+
+void
+rinit_threaded (void)
+{
+    if (!roy_initialized) {
+        roy_initialized = TRUE;
+        rthread_enabled = TRUE;
+
+        rchunk_init__P ();
+        rbuf_auto_init__P ();
+        rthread_init__P ();
+    }
 }
 
 
+
